/*
   Copyright (C) 2000-2002 Ulric Eriksson <ulric@siag.nu>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the Licence, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#define error(p) {perror(p);return;}

static void readdata(int fd, char *b, int n)
{
	int i, j;
	for (i = 0; i < n; i += j) {
		j = read(fd, b+i, n-i);
		if (j < 0) error("Error reading data");
	}
}

static void writedata(int fd, char *b, int n)
{
	int i, j;
	for (i = 0; i < n; i += j) {
		j = write(fd, b+i, n-i);
		if (j < 0) error("Error writing data");
	}
}

static void writestring(int fd, char *b)
{
	writedata(fd, b, strlen(b));
}

/* read a single char */
static int readchar(int fd)
{
	char b[1];
	readdata(fd, b, 1);
	return b[0];
}

/* read a number terminated by non-digit */
static int readno(int fd)
{
	int c, no = 0;

	for (;;) {
		c = readchar(fd);
		if (!isdigit(c)) break;
		no = 10*no + c-'0';
	}
	return no;
}

int sdbd_driver(char *url, char *query,
	int (*callback)(int, char **, void *), void *closure)
{
	int i, sockfd, port, nrow, ncol, colsize;
	struct sockaddr_in serv_addr;
	struct hostent *server;
	char b[4096], host[1024];
	char *p;
	char **coldata;

	p = strstr(url, ":url=");
	if (p == NULL) {
		fprintf(stderr, "No url\n");
		return -1;
	}
	strncpy(host, url, p-url);
	host[p-url] = '\0';
	url = p+5;
	p = strchr(host, ':');
	if (p == NULL) {
		fprintf(stderr, "No port\n");
		return -1;
	}
	*p = '\0';
	port = atoi(p+1);

	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) {
		fprintf(stderr, "Error opening socket\n");
		return -1;
	}
	server = gethostbyname(host);
	if (server == NULL) {
		fprintf(stderr, "Error, no such host\n");
		return -1;
	}
	memset((char *) &serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	memcpy((char *) &serv_addr.sin_addr.s_addr,
	      (char *) server->h_addr, server->h_length);
	serv_addr.sin_port = htons(port);
	if (connect(sockfd, (struct sockaddr *)&serv_addr,
		    sizeof(serv_addr)) < 0) {
		fprintf(stderr, "Error connecting\n");
		return -1;
	}
	writestring(sockfd, url);
	readchar(sockfd);
	writestring(sockfd, query);
	nrow = 0;
	while ((ncol = readno(sockfd)) > 0) {
		coldata = malloc(ncol*sizeof *coldata);
		for (i = 0; i < ncol; i++) {
			colsize = readno(sockfd);
			memset(b, 0, sizeof b);
			readdata(sockfd, b, colsize);
			coldata[i] = strdup(b);
		}
		(*callback)(ncol, coldata, closure);
		for (i = 0; i < ncol; i++) free(coldata[i]);
		free(coldata);
		nrow++;
	}
	close(sockfd);

	return nrow;
}

